import click

@click.group()
def cli():
    pass

@cli.command()
@click.option('--host', default='127.0.0.1')
@click.option('--port', default=5000)
def run(host, port):
    from .globals import current_app
    app = current_app
    
    print(f"Application at http://{host}:{port}")
    
    from wsgiref.simple_server import make_server
    server = make_server(host, port, app)
    server.serve_forever()

@cli.command()
def routes():
    from .globals import current_app
    app = current_app
    
    print("Routes:")
    for (method, path), handler in app.routes.items():
        print(f"{method} {path} -> {handler.__name__}")

def main():
    cli()

if __name__ == '__main__':
    main()


import typing as t
from threading import Lock
from weakref import WeakKeyDictionary

class _AppCtxGlobals:
    __slots__ = ()

    def __init__(self):
        self._data = {}

    def get(self, name, default=None):
        return self._data.get(name, default)

    def pop(self, name, default=None):
        return self._data.pop(name, default)

    def setdefault(self, name, default=None):
        return self._data.setdefault(name, default)

    def __contains__(self, item):
        return item in self._data

    def __getitem__(self, item):
        return self._data[item]

    def __setitem__(self, key, value):
        self._data[key] = value

    def __delitem__(self, key):
        del self._data[key]

    def __iter__(self):
        return iter(self._data)

    def __repr__(self):
        return f"<AppCtxGlobals {self._data!r}>"

class _AppContext:
    __slots__ = ('app', 'g', '_refcnt')

    def __init__(self, app):
        self.app = app
        self.g = app.app_ctx_globals_class()
        self._refcnt = 0

class _RequestContext:
    __slots__ = ('app', 'request', 'session', 'url_adapter', '_implicit_app_ctx_stack', '_preserved')

    def __init__(self, app, environ):
        self.app = app
        self.request = app.request_class(environ)
        self.session = None
        self.url_adapter = None
        self._implicit_app_ctx_stack = []
        self._preserved = False

    def push(self):
        top = _request_ctx_stack.top
        if top is not None and top.preserved:
            top.pop(top._preserved_exc)

        app_ctx = _app_ctx_stack.top
        if app_ctx is None or app_ctx.app != self.app:
            app_ctx = self.app.app_context()
            app_ctx.push()
            self._implicit_app_ctx_stack.append(app_ctx)
        else:
            self._implicit_app_ctx_stack.append(None)

        _request_ctx_stack.push(self)

        if self.session is None:
            session_interface = self.app.session_interface
            self.session = session_interface.open_session(self.app, self.request)
            if self.session is None:
                self.session = session_interface.make_null_session(self.app)

        if self.url_adapter is None:
            self.url_adapter = self.app.create_url_adapter(self.request)

    def pop(self, exc=None):
        app_ctx = self._implicit_app_ctx_stack.pop()
        try:
            if not self._preserved:
                self.app.do_teardown_request(exc)
            request_ctx = _request_ctx_stack.pop()
            if request_ctx is not self:
                raise AssertionError("Popped wrong request context.")
        finally:
            if app_ctx is not None:
                app_ctx.pop(exc)

    def __enter__(self):
        self.push()
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self.pop(exc_value)

class _AppCtxStack:
    __slots__ = ('_stack',)

    def __init__(self):
        self._stack = []

    def push(self, obj):
        self._stack.append(obj)

    def pop(self):
        return self._stack.pop()

    @property
    def top(self):
        if self._stack:
            return self._stack[-1]
        return None

    def __len__(self):
        return len(self._stack)

    def __iter__(self):
        return iter(self._stack)

class _RequestCtxStack(_AppCtxStack):
    __slots__ = ()

_app_ctx_stack = _AppCtxStack()
_request_ctx_stack = _RequestCtxStack()

class _Lookup:
    __slots__ = ('name', 'description')

    def __init__(self, name, description):
        self.name = name
        self.description = description

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        return getattr(obj, f"_get_{self.name}")()

    def __set__(self, obj, value):
        raise AttributeError(f"can't set attribute {self.name!r}")

    def __delete__(self, obj):
        raise AttributeError(f"can't delete attribute {self.name!r}")

def _lookup_req_object(name):
    top = _request_ctx_stack.top
    if top is None:
        raise RuntimeError(f"Working outside of request context: {name}")
    return getattr(top, name)

def _lookup_app_object(name):
    top = _app_ctx_stack.top
    if top is None:
        raise RuntimeError(f"Working outside of application context: {name}")
    return getattr(top, name)

def _find_app():
    top = _app_ctx_stack.top
    if top is None:
        raise RuntimeError("Working outside of application context.")
    return top.app

class _RequestProxy:
    __slots__ = ()

    def _get_request(self):
        return _lookup_req_object('request')

    def __getattr__(self, name):
        return getattr(self._get_request(), name)

    def __setattr__(self, name, value):
        setattr(self._get_request(), name, value)

    def __delattr__(self, name):
        delattr(self._get_request(), name)

    def __getitem__(self, key):
        return self._get_request()[key]

    def __setitem__(self, key, value):
        self._get_request()[key] = value

    def __delitem__(self, key):
        del self._get_request()[key]

    def __contains__(self, key):
        return key in self._get_request()

    def __iter__(self):
        return iter(self._get_request())

    def __len__(self):
        return len(self._get_request())

    def __repr__(self):
        return repr(self._get_request())

class _SessionProxy:
    __slots__ = ()

    def _get_session(self):
        return _lookup_req_object('session')

    def __getattr__(self, name):
        return getattr(self._get_session(), name)

    def __setattr__(self, name, value):
        setattr(self._get_session(), name, value)

    def __delattr__(self, name):
        delattr(self._get_session(), name)

    def __getitem__(self, key):
        return self._get_session()[key]

    def __setitem__(self, key, value):
        self._get_session()[key] = value

    def __delitem__(self, key):
        del self._get_session()[key]

    def __contains__(self, key):
        return key in self._get_session()

    def __iter__(self):
        return iter(self._get_session())

    def __len__(self):
        return len(self._get_session())

    def __repr__(self):
        return repr(self._get_session())

class _GProxy:
    __slots__ = ()

    def _get_g(self):
        return _lookup_app_object('g')

    def __getattr__(self, name):
        return getattr(self._get_g(), name)

    def __setattr__(self, name, value):
        setattr(self._get_g(), name, value)

    def __delattr__(self, name):
        delattr(self._get_g(), name)

    def __getitem__(self, key):
        return self._get_g()[key]

    def __setitem__(self, key, value):
        self._get_g()[key] = value

    def __delitem__(self, key):
        del self._get_g()[key]

    def __contains__(self, key):
        return key in self._get_g()

    def __iter__(self):
        return iter(self._get_g())

    def __len__(self):
        return len(self._get_g())

    def __repr__(self):
        return repr(self._get_g())

class _CurrentAppProxy:
    __slots__ = ()

    def _get_current_app(self):
        return _find_app()

    def __getattr__(self, name):
        return getattr(self._get_current_app(), name)

    def __setattr__(self, name, value):
        setattr(self._get_current_app(), name, value)

    def __delattr__(self, name):
        delattr(self._get_current_app(), name)

    def __repr__(self):
        return repr(self._get_current_app())

request = _RequestProxy()
session = _SessionProxy()
g = _GProxy()
current_app = _CurrentAppProxy()

def has_request_context():
    return _request_ctx_stack.top is not None

def has_app_context():
    return _app_ctx_stack.top is not None

def _get_flashed_messages():
    return session.get('_flashes', [])

def _set_flashed_messages(messages):
    session['_flashes'] = messages

def flash(message, category='message'):
    flashes = session.get('_flashes', [])
    flashes.append((category, message))
    session['_flashes'] = flashes

def get_flashed_messages(with_categories=False):
    flashes = _get_flashed_messages()
    if flashes:
        _set_flashed_messages([])
    if with_categories:
        return flashes
    return [msg for _, msg in flashes]

from .helpers import url_for, abort, redirect, jsonify, render_template
from .helpers import send_file, send_from_directory, make_response


from .request import Request
from .response import Response

class EnvironBuilder:
    def __init__(self, method='GET', path='/', headers=None, data=None, json=None):
        self.method = method
        self.path = path
        self.headers = headers or {}
        self.data = data
        self.json = json

    def get_environ(self):
        environ = {
            'REQUEST_METHOD': self.method,
            'PATH_INFO': self.path,
            'QUERY_STRING': '',
            'SERVER_NAME': 'localhost',
            'SERVER_PORT': '8000',
            'wsgi.version': (1, 0),
            'wsgi.url_scheme': 'http',
            'wsgi.input': None,
            'wsgi.errors': None,
            'wsgi.multithread': False,
            'wsgi.multiprocess': False,
            'wsgi.run_once': False,
        }
        return environ

class ResponseStream:
    def __init__(self):
        self._data = []

    def write(self, data):
        self._data.append(data)

    def get_data(self):
        return b''.join(self._data)


from .app import starexx
from .globals import request, session, g, current_app
from .globals import url_for, abort, redirect, jsonify, render_template_string
from .routing import Router, Rule
from .request import Request
from .response import Response
from .config import Config
from .exceptions import HTTPException, BadRequest, NotFound, InternalServerError
from .middleware import Middleware
from .templating import TemplateEngine
from .signals import signals, request_started, request_finished, got_request_exception
from .sessions import SessionInterface, SecureCookieSession, Session

__version__ = "1.0.5"
__author__ = "Starexx"
__license__ = "MIT"

__all__ = [
    'starexx', 'Request', 'Response', 'Config',
    'request', 'session', 'g', 'current_app',
    'Router', 'Rule',
    'url_for', 'abort', 'redirect', 'jsonify', 'render_template_string',
    'HTTPException', 'BadRequest', 'NotFound', 'InternalServerError',
    'Middleware',
    'TemplateEngine',
    'signals', 'request_started', 'request_finished', 'got_request_exception',
    'SessionInterface', 'SecureCookieSession', 'Session'
]

def create_app(import_name, **kwargs):
    app = starexx(import_name, **kwargs)
    return app

def __getattr__(name):
    if name == '__version__':
        return __version__
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


import json
import datetime
from decimal import Decimal

class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        if isinstance(obj, datetime.date):
            return obj.isoformat()
        if isinstance(obj, Decimal):
            return float(obj)
        return super().default(obj)

class JSONDecoder(json.JSONDecoder):
    pass

class JSONProvider:
    def __init__(self, app=None):
        self.app = app
        self.encoder = JSONEncoder()
        self.decoder = JSONDecoder()

    def dumps(self, obj, **kwargs):
        return self.encoder.encode(obj)

    def loads(self, s, **kwargs):
        return self.decoder.decode(s)

    def response(self, data):
        from .response import Response
        return Response(self.dumps(data), content_type='application/json')

json_provider = JSONProvider()


import typing as t
import re
import warnings
from threading import Lock
from urllib.parse import quote
from weakref import WeakKeyDictionary

class Rule:
    __slots__ = ('rule', 'methods', 'endpoint', 'defaults', 'subdomain', 'strict_slashes', 'merge_slashes', 'redirect_to', 'alias', 'host')

    def __init__(self, rule, methods=None, endpoint=None, defaults=None, subdomain=None, strict_slashes=True, merge_slashes=True, redirect_to=None, alias=None, host=None):
        self.rule = rule
        self.methods = methods
        self.endpoint = endpoint
        self.defaults = defaults
        self.subdomain = subdomain
        self.strict_slashes = strict_slashes
        self.merge_slashes = merge_slashes
        self.redirect_to = redirect_to
        self.alias = alias
        self.host = host

    def compile(self):
        return RuleCompiler().compile(self.rule)

    def match(self, path):
        return self.compile().match(path)

    def build(self, values, append_unknown=True):
        return self.compile().build(values, append_unknown)

    def __str__(self):
        return self.rule

    def __repr__(self):
        return f"<Rule '{self.rule}' -> {self.endpoint}>"

class RuleCompiler:
    __slots__ = ('_cache', '_lock')

    def __init__(self):
        self._cache = {}
        self._lock = Lock()

    def compile(self, rule):
        with self._lock:
            if rule in self._cache:
                return self._cache[rule]

            regex_parts = []
            converter_parts = []
            is_static = True
            pos = 0
            rule_len = len(rule)

            while pos < rule_len:
                match = re.search(r'<(.+?)>', rule[pos:])
                if not match:
                    regex_parts.append(re.escape(rule[pos:]))
                    break

                start, end = match.span()
                regex_parts.append(re.escape(rule[pos:pos + start]))
                pos += start

                converter = match.group(1)
                if ':' in converter:
                    converter_name, arg = converter.split(':', 1)
                    if converter_name == 'int':
                        regex = r'\d+'
                    elif converter_name == 'float':
                        regex = r'\d+\.\d+'
                    elif converter_name == 'path':
                        regex = r'.*?'
                    elif converter_name == 'uuid':
                        regex = r'[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}'
                    else:
                        regex = r'[^/]+'
                    converter_parts.append((converter_name, arg))
                else:
                    regex = r'[^/]+'
                    converter_parts.append((converter, None))

                regex_parts.append(f'({regex})')
                pos += end - start
                is_static = False

            regex_pattern = '^' + ''.join(regex_parts)
            if not rule.endswith('/'):
                regex_pattern += '/?'

            compiled_rule = CompiledRule(regex_pattern, converter_parts, is_static)
            self._cache[rule] = compiled_rule
            return compiled_rule

class CompiledRule:
    __slots__ = ('regex', 'converters', 'is_static', '_compiled_regex')

    def __init__(self, regex, converters, is_static):
        self.regex = regex
        self.converters = converters
        self.is_static = is_static
        self._compiled_regex = re.compile(regex)

    def match(self, path):
        match = self._compiled_regex.match(path)
        if not match:
            return None, {}

        groups = match.groups()
        values = {}
        for (name, arg), value in zip(self.converters, groups):
            if name == 'int':
                try:
                    values[name] = int(value)
                except ValueError:
                    return None, {}
            elif name == 'float':
                try:
                    values[name] = float(value)
                except ValueError:
                    return None, {}
            elif name == 'path':
                values[name] = value
            else:
                values[name] = value

        return self, values

    def build(self, values, append_unknown=True):
        path = self.regex
        for (name, arg), value in zip(self.converters, values.values()):
            path = path.replace(f'({re.escape(str(value))})', str(value), 1)
        return path

class Router:
    __slots__ = ('rules', '_rules_by_endpoint', '_rules_by_method', '_lock')

    def __init__(self):
        self.rules = []
        self._rules_by_endpoint = {}
        self._rules_by_method = {}
        self._lock = Lock()

    def add_rule(self, rule, view_func):
        with self._lock:
            self.rules.append((rule, view_func))
            self._rules_by_endpoint[rule.endpoint] = (rule, view_func)
            
            for method in rule.methods:
                if method not in self._rules_by_method:
                    self._rules_by_method[method] = []
                self._rules_by_method[method].append((rule, view_func))

    def match(self, method, path):
        if method not in self._rules_by_method:
            return None, {}

        for rule, view_func in self._rules_by_method[method]:
            matched_rule, values = rule.match(path)
            if matched_rule:
                return view_func, values

        return None, {}

    def build(self, endpoint, values=None, method=None):
        if endpoint not in self._rules_by_endpoint:
            return None

        rule, view_func = self._rules_by_endpoint[endpoint]
        return rule.build(values or {})

    def create_url_adapter(self, request):
        return URLAdapter(self, request)

class URLAdapter:
    __slots__ = ('router', 'request')

    def __init__(self, router, request):
        self.router = router
        self.request = request

    def build(self, endpoint, values=None, method=None, force_external=False):
        return self.router.build(endpoint, values, method)

    def match(self, path=None, method=None):
        if path is None:
            path = self.request.path
        if method is None:
            method = self.request.method
        return self.router.match(method, path)

class Blueprint:
    __slots__ = ('name', 'import_name', 'url_prefix', 'subdomain', 'url_defaults', 'root_path', 'deferred_functions', 'rules')

    def __init__(self, name, import_name, url_prefix=None, subdomain=None, url_defaults=None):
        self.name = name
        self.import_name = import_name
        self.url_prefix = url_prefix
        self.subdomain = subdomain
        self.url_defaults = url_defaults or {}
        self.root_path = self._find_root_path()
        self.deferred_functions = []
        self.rules = []

    def _find_root_path(self):
        module = __import__(self.import_name)
        return module.__file__

    def route(self, rule, **options):
        def decorator(f):
            endpoint = options.pop("endpoint", f.__name__)
            self.add_url_rule(rule, endpoint, f, **options)
            return f
        return decorator

    def add_url_rule(self, rule, endpoint=None, view_func=None, **options):
        if endpoint is None:
            endpoint = view_func.__name__
            
        if self.url_prefix is not None:
            rule = self.url_prefix + rule
            
        options['endpoint'] = endpoint
        self.rules.append((rule, view_func, options))

    def register(self, app, options):
        url_prefix = options.get('url_prefix') or self.url_prefix
        for rule, view_func, opts in self.rules:
            app.add_url_rule(rule, view_func=view_func, **opts)